#!/usr/bin/env python
#
# Copyright (C) 2012 W. Trevor King <wking@drexel.edu>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program.  If not, see
# <http://www.gnu.org/licenses/>.

"""Extract the printable ASCII range of Unicode characters to WOFF.

This is useful for generating smaller font packages for use with CSS's
`@font-face`.
"""

import os.path as _os_path

import fontforge as _fontforge


__version__ = '0.1'


def convert(input_file, output_file=None, ranges=[(0x20,0x7e)],
            targets=[(0x20,0x7e)], **kwargs):
    if output_file is None:
        base,ext = _os_path.splitext(input_file)
        output_file = '{}-reduced.woff'.format(base)
    if len(ranges) != len(targets):
        raise ValueError(
            'range/target missmatch: ranges: {}, targets: {}.'.format(
                ranges, targets))
    i = _fontforge.open(input_file)
    o = _fontforge.font()
    mappings = []
    copy_space = False
    for ((istart,istop),(ostart,ostop)) in zip(ranges, targets):
        i.selection.select(('ranges',), istart, istop)
        i.copy()
        o.selection.select(('ranges',), ostart, ostop)
        o.paste()
        mappings.append((istart,istop,ostart,ostop))
        if istart <= 0x20 and istop >= 20:
            copy_space = True  # HACK: FF doesn't like spaces?
    if copy_space:
        i.selection.select('space')
        i.copy()
        o.selection.select('space')
        o.paste()
    for attr in ['comment', 'copyright', 'encoding', 'familyname',
                 'fontname', 'fullname', 'sfntRevision', 'sfnt_names',
                 'userdata', 'version', 'woffMetadata']:
        value = getattr(i, attr)
        if attr in ['fontname', 'fullname'] and value:
            value = 'Reduced{}'.format(value)
        setattr(o, attr, value)
    fontlog = ('Reduced to Unicode characters 0x{:x} through 0x{:x} by '
               'font-reduce.py version {}, '
               '`http://blog.tremily.us/posts/font-reduce/`.'
               ).format(ranges[0][0], ranges[0][1], __version__)
    if len(mappings) > 1:
        fontlog += '\n\nAdditional ranges: {}'.format(', '.join(
                '0x{:x}-0x{:x} -> 0x{:x}-0x{:x}'.format(*mapping)
                for mapping in mappings[1:]))
    if i.fontlog:
        fontlog = '{}\n\n{}'.format(i.fontlog.rstrip('\n'), fontlog)
    o.fontlog = fontlog

    if not o.woffMetadata:
        o.woffMetadata = generate_woff_metadata(o, **kwargs)
    print o.woffMetadata
    print [a for a in dir(o) if 'lic' in a.lower()]

    # the `PfEd-comments` flag is required for Fontforge to save
    # `.comment` and `.fontlog`.
    o.generate(output_file, flags=('PfEd-comments',))


def generate_woff_metadata(font, license=None, source=None):
    metadata = [
        '<?xml version="1.0" encoding="UTF-8"?>',
        '<metadata version="1.0">',
        '  <uniqueid id="us.tremily.font-reduce.{}.{}.{}/>'.format(
            __version__, font.fontname, font.version),
        '  <description>',
        '    <text lang="en">{}</text>'.format(font.fontlog),
        '  </description>',
        ]
    if source:
        metadata.extend([
                '  <credits>',
                '    <credit',
                '        name="Original source for this font"',
                '        url="{}"'.format(source),
                '  </credits>',
                ])
    if license:
        metadata.extend([
                '  <license>',
                '    <text lang="en">{}</text>'.format(license),
                '  </license>',
                ])
    if font.copyright:
        metadata.extend([
                '  <copyright>',
                '    <text lang="en">{}</text>'.format(font.copyright),
                '  </copyright>',
                ])
    metadata.extend([
            '</metadata>',
            '',  # for trailing endline
            ])
    return '\n'.join(metadata)


if __name__ == '__main__':
    import argparse

    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        '-l', '--license', help='override font license text')
    parser.add_argument(
        '-s', '--source', help='override font source URL')
    parser.add_argument(
        '-r', '--range', action='append', default=['0x20,0x7e'],
        help=('add an additional range of indices to copy from the original '
              "font (e.g '0x7f,0xff')"))
    parser.add_argument(
        '-t', '--target', action='append', default=['0x20,0x7e'],
        help=('add an additional range of indices to copy to the reduced font '
              "(e.g '0x7f,0xff')"))
    parser.add_argument(
        '-o', '--output', help='override path to output file')
    parser.add_argument(
        'font', nargs='+', help='path to source font')

    args = parser.parse_args()

    ranges = []
    for r in args.range:
        start,stop = [int(x, 0) for x in r.split(',')]
        ranges.append((start,stop))
    targets = []
    for r in args.target:
        start,stop = [int(x, 0) for x in r.split(',')]
        targets.append((start,stop))
    
    for font in args.font:
        convert(font, output_file=args.output, ranges=ranges, targets=targets,
                license=args.license, source=args.source)
